<?php
/**
 * masteruser.php
* @Copyright Copyright (C) 2010- Spiral Scripts
* @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
******/


// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

//jimport('joomla.event.plugin');
jimport('joomla.version');

/**
 * Plugin that checks to see if the password entered belongs to a master user,
 * allowing the administrators or nominated admin to log in as any user
 *
 */
class plgAuthenticationMasterUser extends JPlugin
{
	var $_plugin;
	var $_params;
	
    /**
     * Constructor
     *
	 * @param 	object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
     */
    function plgAuthenticationMasterUser(& $subject, $config) {
        parent::__construct($subject, $config);
		
        $version = new JVersion();
	    if($version->RELEASE == '1.5')
	    {
		    jimport('joomla.event.plugin');
	    	$this->_plugin 	=  JPluginHelper::getPlugin('authentication','masteruser');
	    	$this->_params 	= new JParameter($this->_plugin->params);
		}
		else
		{
			jimport('joomla.plugin.plugin');
			$this->_params =& $this->params;
		}
		
    }

    /**
     * This method handles any  authentication and reports back to the subject
     * The method checks to see if the password entered matches the master user password
     *
     * @access    public
	 * @access	public
	 * @param   array 	$credentials Array holding the user credentials
	 * @param 	array   $options     Array of extra options
     * @param   object  $response    Authentication response object
     * @return    boolean
     * @since 1.5
     */
    function onAuthenticate( $credentials, $options, &$response )
    {
		jimport('joomla.user.helper');

		/*
		 */
		$response->status = JAUTHENTICATE_STATUS_FAILURE;
		$response->error_message = 'Master User Authentication Failed';

		/*
		 * check IP address
		 */
		$restrict_ip = $this->_params->get('restrict_ip', '0');
		if ($restrict_ip == '1') {
			$my_ip = "," . trim($_SERVER['REMOTE_ADDR']) . ",";
			$ip_list = trim($this->_params->get('ip_addresses',''));
 			$ip_list = "," . str_replace(" ","",$ip_list) . ",";
			
			if (strpos($ip_list, $my_ip) === false) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'Master User IP Address Not Allowed';
				return (false);
			}
		}
	
		/*
		 * fail authentication if no username or password
		 */
		if (empty($credentials['password']) || empty($credentials['username'])) {
			$response->status = JAUTHENTICATE_STATUS_FAILURE;
			$response->error_message = 'Either Username or User was not specified';
			return (false);
		}
			/*
			 * Check  if the user exists
			 */	
			$db =& JFactory::getDBO();
			$query = 'SELECT id, gid FROM #__users WHERE username=' . $db->quote($credentials['username']); 
			$db->setQuery($query);
			$result = $db->loadObject();
			
			/*
			 * If username does not exist, fail authentication
			 */
			if (!$result) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'User does not exist';
				return (false);
			}
			/* Check it's not an Administrator or a Super Administrator
			 */
			if (($result->gid == 24) || ($result->gid == 25)) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'Cannot log in as Administrator or Super Adminstrator';
				return (false);
			}
				
				
				/*
				 * To authenticate the password should be equal
				 * to the Master User Password - first find eligible users.
				 */
			  $query = ''; 
			  if ($this->_params->get('enable_all','0') == '1')
			  {
			     $query = 'SELECT id, username, gid, password FROM #__users WHERE ((gid = 24) OR (gid = 25))'; 
			  }
			  else
			  {
				  $users = trim($this->_params->get('master_ids',''));
				  $users = str_replace(' ','',$users);
				  $user_array = explode(',',$users);
			      JArrayHelper::toInteger( $user_array );
				  $whereString = implode(') OR (id = ',$user_array);
				  
			      $query = 'SELECT id, username, gid, password FROM #__users WHERE (((gid = 24) OR (gid = 25)) AND ((id = '.$whereString.')))'; 
			  }
			  $db->setQuery($query);
			  $adminRows = $db->loadObjectList();
			  

			  
			  if( $adminRows)
			  {
				  
				foreach($adminRows AS $adminRow)
				{
					$parts	= explode( ':', $adminRow->password );
					$crypt	= $parts[0];
					$salt	= @$parts[1];
					$testcrypt = JUserHelper::getCryptedPassword($credentials['password'], $salt);


					
				   if($crypt == $testcrypt){
				 
					$user = JUser::getInstance($result->id); // Add user results
					$response->email = $user->email;
					$response->fullname = $user->name;
					$response->status = JAUTHENTICATE_STATUS_SUCCESS;
					$response->error_message = '';
					break;
				   }
				}
			  }//end if adminRows
					
			
    }
	
   /**
     * This method handles any  authentication and reports back to the subject
     * The method checks to see if the password entered matches the master user password
     *
     * @access    public
	 * @access	public
	 * @param   array 	$credentials Array holding the user credentials
	 * @param 	array   $options     Array of extra options
     * @param   object  $response    Authentication response object
     * @return    boolean
     * @since 1.6
     */
    function onUserAuthenticate( $credentials, $options, &$response )
    {
		jimport('joomla.user.helper');
		
	
		/*
		 */
		$response->status = JAUTHENTICATE_STATUS_FAILURE;
		$response->error_message = 'Master User Authentication Failed';
				

		/*
		 * check IP address
		 */
		$restrict_ip = $this->_params->get('restrict_ip', '0');
		if ($restrict_ip == '1') {
			$my_ip = "," . trim($_SERVER['REMOTE_ADDR']) . ",";
			$ip_list = trim($this->_params->get('ip_addresses',''));
 			$ip_list = "," . str_replace(" ","",$ip_list) . ",";
			
			if (strpos($ip_list, $my_ip) === false) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'Master User IP Address Not Allowed';
				return (false);
			}
		}
		
		/*
		 * fail authentication if no username or password
		 */
		if (empty($credentials['password']) || empty($credentials['username'])) {
			$response->status = JAUTHENTICATE_STATUS_FAILURE;
			$response->error_message = 'Either Username or User was not specified';
			return (false);
		}
		
		
	
		/*
		 * Check  if the user exists
		 */	
		$db =& JFactory::getDBO();
		$query = 'SELECT id FROM #__users WHERE username=' . $db->quote($credentials['username']); 
		$db->setQuery($query);
		$result = $db->loadObject();
		
		
		
		/*
		 * If username does not exist, fail authentication
		 */
		if (!$result) {
			$response->status = JAUTHENTICATE_STATUS_FAILURE;
			$response->error_message = 'User does not exist';
			return (false);
		}
		/* Check it's not an Administrator or a Super Administrator
		 */
		
		$query ='SELECT group_id FROM #__user_usergroup_map WHERE user_id='. (int)$result->id;
		$db->setQuery($query);
		$groups = $db->loadResultArray();
		
		
		$adminGroups = (array)$this->_params->get('admin_usergroups', array(7,8));
		JArrayHelper::toInteger($adminGroups);
		
		
		
		foreach($adminGroups as $adminGroup)
		{
			if(in_array($adminGroup, $groups))
			{
			  $response->status = JAUTHENTICATE_STATUS_FAILURE;
			  $response->error_message = 'Login failed';
			  return false;
			}
		}
		
		
				
		  /*
		   * To authenticate the password should be equal
		   * to the Master User Password - first find eligible users.
		   */
		$query = ''; 
		if ($this->_params->get('enable_all','0') == '1')
		{
		   $query = 'SELECT id, username, password, m.group_id AS group_id FROM #__users INNER JOIN #__user_usergroup_map AS m WHERE m.group_id  IN('. implode(',',$adminGroups).')'; 
		   
		   
		}
		else
		{
			$users = trim($this->_params->get('master_ids',''));
			$users = str_replace(' ','',$users);
			$user_array = explode(',',$users);
			JArrayHelper::toInteger( $user_array );
			$whereString = implode(') OR (id = ',$user_array);
			
			$query = 'SELECT id, username, password, m.group_id AS group_id FROM #__users INNER JOIN #__user_usergroup_map AS m WHERE ((m.group_id  IN('. implode(',',$adminGroups).')) AND (id = '.$whereString.'))'; 
		}
		$db->setQuery($query);
		$adminRows = $db->loadObjectList();
		
		
		
		if( $adminRows)
		{
			
			
			
		  foreach($adminRows AS $adminRow)
		  {
			  $parts	= explode( ':', $adminRow->password );
			  $crypt	= $parts[0];
			  $salt	= @$parts[1];
			  $testcrypt = JUserHelper::getCryptedPassword($credentials['password'], $salt);


			  
			 if($crypt == $testcrypt){
		   
			  $user = JUser::getInstance($result->id); // Add user results
			  $response->email = $user->email;
			  $response->fullname = $user->name;
			  $response->status = JAUTHENTICATE_STATUS_SUCCESS;
			  $response->error_message = '';
			  break;
			 }
		  }
		}//end if adminRows		
		
		//$mainframe =& JFactory::getApplication('site');
		//$mainframe->redirect('index.php', '<strong>this line reached - adminrows '.'</strong>');
		
		
	}//end onUserAuthenticate
}
?>